"""
Download SINASC data from DATASUS FTP server
Created on 01/11/17
by fccoelho
license: GPL V3 or Later
"""
from typing import Union

from pysus.ftp import CACHEPATH
from pysus.ftp.databases.sinasc import SINASC

sinasc = SINASC().load()


def download(
    groups: Union[str, list],
    states: Union[str, list],
    years: Union[str, list, int],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Downloads data directly from Datasus ftp server
    :param groups: either DN, DNR or both
    :param states: two-letter state identifier: MG == Minas Gerais,
                   can be a list
    :param years: years to download
    :return: list of downloaded files
    """
    files = sinasc.get_files(groups, uf=states, year=years)
    return sinasc.download(files, local_dir=data_dir)


def get_available_years(state):
    years = list(set([f.name[-2:] for f in sinasc.files]))
    files = set(sinasc.get_files(["DN", "DNR"], uf=state, year=years))

    def sort_year(file):
        _, year = sinasc.format(file)
        return int(year)

    return sorted(files, key=sort_year)



import pytest

import datetime
import os
from pathlib import Path
import shutil
import unittest
from glob import glob
from pathlib import Path

import numpy as np
import pandas as pd

from pysus.online_data.SINAN import (
    download,
    list_diseases,
    metadata_df
)
from pysus.online_data import FTP_SINAN, parquets_to_dataframe
from pysus.preprocessing.sinan import read_sinan_dbf

PATH_ROOT = Path(__file__).resolve().parent


class TestSINANClass(unittest.TestCase):
    data_path = '/tmp/pysus'
    d1 = 'Raiva Humana'
    r1 = [
        'RAIVBR07.parquet',
        'RAIVBR08.parquet',
        'RAIVBR09.parquet',
        'RAIVBR10.parquet',
        'RAIVBR11.parquet',
        'RAIVBR12.parquet',
        'RAIVBR13.parquet',
        'RAIVBR14.parquet',
        'RAIVBR15.parquet',
        'RAIVBR16.parquet',
        'RAIVBR17.parquet',
        'RAIVBR18.parquet',
        'RAIVBR19.parquet',
    ]

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_list_all_diseases(self):
        all_diseases = list(FTP_SINAN.diseases.keys())
        self.assertIn('Dengue', all_diseases)
        self.assertIn('Zika', all_diseases)
        self.assertIn('Chikungunya', all_diseases)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download(self):
        files = download(self.d1, [7,8,9], data_path=self.data_path)
        self.assertEqual(len(files), 3)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_read_dataframe(self):
        df = parquets_to_dataframe(Path(self.data_path)/self.r1[0])
        self.assertIsInstance(df, pd.DataFrame)
        self.assertEqual(df.shape, (110, 94))
    
    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_metadata_dataframe(self):
        df = metadata_df('Raiva Humana')
        self.assertIsInstance(df, pd.DataFrame)
        self.assertEqual(df.shape, (68, 7))


class TestSINANDownload(unittest.TestCase):
    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download(self):
        df = parquets_to_dataframe(download(years=2007, disease='Botulismo'))
        self.assertIsInstance(df, pd.DataFrame)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_filename_only(self):
        fname = download(years=2015, disease='Botulismo')
        self.assertIsInstance(fname, str)
        self.assertTrue(os.path.exists(fname))
        shutil.rmtree(fname, ignore_errors=True)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_fetch_viol_dom(self):
        df = parquets_to_dataframe(download(years=2011, disease='Hantavirose'))
        self.assertIsInstance(df, pd.DataFrame)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_fetch_cancer_prelim(self):
        df = parquets_to_dataframe(download(years=2022, disease='Cancer'))
        self.assertIsInstance(df, pd.DataFrame)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_fetch_sifilis(self):
        self.assertRaises(
            Exception, download(years=2021, disease='Sífilis Adquirida')
        )

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_fetch_sifilis_gestante(self):
        df = parquets_to_dataframe(download(years=2021, disease='Sífilis em Gestante'))
        self.assertIsInstance(df, pd.DataFrame)

    @pytest.mark.timeout(5)
    def test_lista_agravos(self):
        lista = list_diseases()
        self.assertIsInstance(lista, dict)
        self.assertGreater(len(lista), 0)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_chunked_df_size(self):
        df1 = parquets_to_dataframe(download(years=2018, disease='Chikungunya'))
        s1 = len(df1)
        del df1
        fn = download(years=2018, disease='Chikungunya')
        for i, f in enumerate(glob(f'{fn}/*.parquet')):
            if i == 0:
                df2 = pd.read_parquet(f)
            else:
                df2 = pd.concat([df2, pd.read_parquet(f)], ignore_index=True)
        self.assertEqual(s1, df2.shape[0])
        shutil.rmtree(fn, ignore_errors=True)


class TestSinanDBF(unittest.TestCase):
    dbf_name = PATH_ROOT / 'EPR-2016-06-01-2016.dbf'

    @pytest.mark.timeout(5)
    def test_read_dbf(self):
        df = read_sinan_dbf(self.dbf_name, encoding='latin-1')
        self.assertTrue(self.dbf_name.exists())
        self.assertIsInstance(df, pd.DataFrame)
        for cname in df.columns:
            if cname.startswith('DT_'):
                self.assertIsInstance(df[cname][0], datetime.date)
            elif cname.startswith('SEM'):
                self.assertLessEqual(df[cname][0], 52)
                self.assertIsInstance(df[cname][0], (int, np.int64))
            elif cname.startswith(('NU', 'ID')):
                if cname == 'ID_AGRAVO':
                    continue
                self.assertIsInstance(
                    df[cname][0],
                    (int, float, np.int64),
                    msg='Failed on column {}, type:{}'.format(
                        cname, type(df[cname][0])
                    ),
                )

    @pytest.mark.timeout(5)
    def test_type_convertion(self):
        df = read_sinan_dbf(self.dbf_name, encoding='latin-1')
        self.assertTrue(self.dbf_name.exists())
        assert not all(df.dtypes == 'object')


if __name__ == '__main__':
    unittest.main()



import unittest
import pytest

from pysus.online_data.SIH import download
from pysus.online_data import parquets_to_dataframe as to_df

class SIHTestCase(unittest.TestCase):
    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_pre_2008(self):
        df = to_df(download("AC", 2006, 12))
        assert not df.empty

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_2008(self):
        df = to_df(download("SE", 2008, 6))
        assert not df.empty

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_2010(self):
        df = to_df(download("SE", 2010, 6))
        assert not df.empty

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_2019(self):
        df = to_df(download("SE", 2019, 6))
        assert not df.empty


if __name__ == "__main__":
    unittest.main()



__author__ = "fccoelho"

import unittest
import pytest

import pandas as pd

from pysus.online_data.CIHA import download
from pysus.online_data import parquets_to_dataframe

unittest.skip("too slow to run om travis")


class SIHTestCase(unittest.TestCase):
    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_CIH(self):
        files = download("mg", 2011, 7)
        df = parquets_to_dataframe(files)
        self.assertGreater(len(df), 0)
        self.assertIn("DIAG_PRINC", df.columns)
        self.assertIsInstance(df, pd.DataFrame)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_CIHA(self):
        files = download("MG", 2013, 10)
        df = parquets_to_dataframe(files)
        self.assertGreater(len(df), 0)
        self.assertIn("DIAG_PRINC", df.columns)
        self.assertIsInstance(df, pd.DataFrame)


if __name__ == "__main__":
    unittest.main()



"""
Downloads SIA data from Datasus FTP server
Created on 21/09/18
by fccoelho
Modified on 22/11/22
by bcbernardo
license: GPL V3 or Later
"""
from pprint import pprint
from typing import Dict, Tuple, Union

from pysus.ftp import CACHEPATH
from pysus.ftp.databases.sia import SIA

sia = SIA().load()


group_dict: Dict[str, Tuple[str, int, int]] = {
    "PA": ("Produção Ambulatorial", 7, 1994),
    "BI": ("Boletim de Produção Ambulatorial individualizado", 1, 2008),
    "AD": ("APAC de Laudos Diversos", 1, 2008),
    "AM": ("APAC de Medicamentos", 1, 2008),
    "AN": ("APAC de Nefrologia", 1, 2008),
    "AQ": ("APAC de Quimioterapia", 1, 2008),
    "AR": ("APAC de Radioterapia", 1, 2008),
    "AB": ("APAC de Cirurgia Bariátrica", 1, 2008),
    "ACF": ("APAC de Confecção de Fístula", 1, 2008),
    "ATD": ("APAC de Tratamento Dialítico", 1, 2008),
    "AMP": ("APAC de Acompanhamento Multiprofissional", 1, 2008),
    "SAD": ("RAAS de Atenção Domiciliar", 1, 2008),
    "PS": ("RAAS Psicossocial", 1, 2008),
}


def show_datatypes():
    pprint(group_dict)


def download(
    states: Union[str, list],
    years: Union[str, list, int],
    months: Union[str, list, int],
    groups: Union[str, list],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Download SIASUS records for state year and month and returns dataframe
    :param states: 2 letter state code, can be a list
    :param years: 4 digit integer, can be a list
    :param months: 1 to 12, can be a list
    :param data_dir: whether to cache files locally. default is True
    :param group: SIA groups. For all groups, refer to `sia.groups`
    :return: list of downloaded files
    """
    files = sia.get_files(
        groups=groups, ufs=states, years=years, months=months
    )
    downloaded = []
    for file in files:
        downloaded.append(file.download(local_dir=data_dir))
    return downloaded



"""
Downloads SIH data from Datasus FTP server
Created on 21/09/18
by fccoelho
license: GPL V3 or Later
"""
from typing import Union

from pysus.ftp import CACHEPATH
from pysus.ftp.databases.sih import SIH

sih = SIH().load()


def download(
    states: Union[str, list],
    years: Union[str, list, int],
    months: Union[str, list, int],
    groups: Union[str, list],
    data_dir: str = CACHEPATH,
) -> list:
    """
    Download SIH records for state, year and month
    :param states: 2 letter state code, can be a list
    :param years: 4 digit integer, can be a list
    :param months: 1 to 12, can be a list
    :param groups: the groups of datasets to be downloaded.
                   See `sih.groups`
    :param data_dir: Directory where parquets will be downloaded.
    :return: list with the downloaded files
    """
    files = sih.get_files(
        groups=groups, ufs=states, months=months, years=years
    )
    downloaded = []
    for file in files:
        downloaded.append(file.download(local_dir=data_dir))
    return downloaded



__author__ = "fccoelho"

import unittest
import pytest

from pysus.online_data.sinasc import download, get_available_years, sinasc
from pysus.online_data import parquets_to_dataframe as to_df


class TestDownload(unittest.TestCase):
    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_new(self):
        df = to_df(download("SE", 2015))
        self.assertIn("IDADEMAE", df.columns)
        self.assertGreater(len(df), 0)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_download_old(self):
        df = to_df(download("AL", 1994)[0]) #[0] bc there is a file duplicity in the ftp sever
        self.assertIn("IDADE_MAE", df.columns)
        self.assertGreater(len(df), 0)

    @pytest.mark.skip(reason="This test takes too long")
    @pytest.mark.timeout(5)
    def test_get_available_years(self):
        files = get_available_years("AC")
        self.assertIn("1996", [sinasc.format(file)[1] for file in files])


if __name__ == "__main__":
    unittest.main()
